local addon = LibStub("AceAddon-3.0"):GetAddon("Langoustine")
local tanks = addon:NewModule("TrankFrames", "AceEvent-3.0","AceConsole-3.0")
local LSM = LibStub:GetLibrary("LibSharedMedia-3.0")

local TankHeader,TankAnchor = nil, nil

local function ConfigureFunction(uf)
	local db = addon.db.profile.TankOptions
	print(uf:GetAttribute("unit"))
	uf:SetAttribute("*type1", "target")
	-- we switch the tank unit it in a vechicle
	uf:SetAttribute("toggleForVehicle",true)
	uf:RegisterForClicks("AnyUp")
	--uf:SetScript("OnAttributeChanged", nil) -- TODO add calback to refresh unit information if it has switched!
	tanks:SetupFrame(uf)
	-- Setup target
	uf.target = CreateFrame("Button",uf:GetName().."Target",uf,"SecureActionButtonTemplate")
	uf.target:SetAttribute("useparent-unit", true)
	uf.target:SetAttribute("unitsuffix", "target")
	uf.target:SetAttribute("*type1","target")
	uf.target:RegisterForClicks("AnyUp")	
	tanks:SetupFrame(uf.target)
	if not addon.db.profile.TankShowTarget then
		uf.target:Hide()
	end
	uf.target:SetPoint("RIGHT",uf,"RIGHT",db.unitWidth+3,0)
	-- setup ToT
	uf.tot = CreateFrame("Button",uf:GetName().."TargetTarget",uf,"SecureActionButtonTemplate")
	uf.tot:SetAttribute("useparent-unit", true)
	uf.tot:SetAttribute("unitsuffix", "targettarget")
	uf.tot:SetAttribute("*type1","target")
	uf.tot:RegisterForClicks("AnyUp")
	tanks:SetupFrame(uf.tot)
	uf.tot:SetPoint("RIGHT",uf.target,"RIGHT",db.unitWidth+3,0)	
	if not addon.db.profile.TankShowTargetTarget then
		uf.tot:Hide()
	end
	-- Setup ClickCastFrames
	ClickCastFrames = ClickCastFrames or {}
	ClickCastFrames[uf] = true
	ClickCastFrames[uf.target] = true
	ClickCastFrames[uf.tot] = true
	tinsert(tanks.tankFrames, uf)
end

function tanks:OnInitialize()
	local db = addon.db.profile.TankOptions
	self.tankFrames = {}
	self.blizzTanks = {}
	self.localTanks = {}
	self.timer = 0
end
function tanks:SetupUI()
	local db = addon.db.profile.TankOptions
	TankAnchor = CreateFrame("Frame","LTA",UIParent)
	TankAnchor:SetWidth(db.unitWidth)
	TankAnchor:SetHeight(15)
	TankAnchor:SetPoint("CENTER",UIParent,"CENTER",0,0)
	TankAnchor.label = TankAnchor:CreateFontString(nil,"OVERLAY","GameFontWhite")
	TankAnchor.label:SetAllPoints(TankAnchor)
	TankAnchor.label:SetText("Tank Anchor")
	TankAnchor.label:Show()
	TankAnchor:SetBackdrop(
		{bgFile = "Interface/Tooltips/UI-Tooltip-Background", 
	 	edgeFile = "Interface/Tooltips/UI-Tooltip-Border", 
	 	tile = false, tileSize = 16, edgeSize = 8, 
	 	insets = { left = 2, right = 2, top = 2, bottom = 2 }})
	
	TankAnchor:SetBackdropColor(0,0,0,1)
	TankAnchor:EnableMouse(true)
	TankAnchor:SetMovable(true)
	TankAnchor:RegisterForDrag("LeftButton")
	TankAnchor:SetScript("OnDragStart", function(self) if not InCombatLockdown() then self:StartMoving() end end)
	TankAnchor:SetScript("OnDragStop", function(self) if not InCombatLockdown() then self:StopMovingOrSizing() end end)
	TankAnchor:Show()

	TankHeader = CreateFrame("Frame","LTH",TankAnchor,"SecureRaidGroupHeaderTemplate")
	TankHeader.initialConfigFunction = ConfigureFunction
	TankHeader:SetAttribute("template","SecureUnitButtonTemplate")
	TankHeader:SetWidth(db.unitWidth)
	TankHeader:SetHeight(15)
	TankHeader:SetAttribute("minheight",15)
	TankHeader:SetAttribute("sortMethod","NAME")
	TankHeader:SetAttribute("sortDir","DESC")
	TankHeader:SetAttribute("unitsPerColumn",db.MaxTanks)
	TankHeader:SetAttribute("columnSpacing",2)
	if (addon.db.profile.TankOptions.growth == "Down") then
		TankHeader:SetAttribute("point", "BOTTOM")
		TankHeader:SetPoint("TOP", TankAnchor,"BOTTOM", 0, 0)	
	else
		TankHeader:SetAttribute("point", "TOP")
		TankHeader:SetPoint("BOTTOM", TankAnchor,"TOP", 0, 0)	
	end
end

function tanks:OnEnable()
	self:SetupUI()
	TankHeader:Show()
	self:RegisterEvent("RAID_ROSTER_UPDATE","RosterUpdate")
	self:RosterUpdate()
	self:RegisterChatCommand("pt","AddTank")
end

function tanks:OnDisable()
	self:UnregisterEvent("RAID_ROSTER_UPDATE")
end
local healthFormat="%d%%"
function tanks:UpdateTankFrame(frame)
	if not frame then return end
	local unit = SecureButton_GetUnit(frame)
	if unit and UnitExists(unit) then
		local name = UnitName(unit)
		frame.name:SetText(name)
		local cur, max = UnitHealth(unit) or 0, UnitHealthMax(unit) or 0
		local perc = ceil((cur / max) * 100)
		frame.health:SetText(format(healthFormat,perc))
		frame.bar:SetValue(perc)
		local icon = GetRaidTargetIndex(unit)
		if icon then
			SetRaidTargetIconTexture(frame.icon, icon)
			frame.icon:Show()
		else
			frame.icon:Hide()
		end
		frame:Show()
	else
		frame:Hide()
	end
end

function tanks:OnUpdate(elapsed)
	self.timer = self.timer + elapsed
	if self.timer > 0.3 then -- we update tank frames every 1/3rd of a sec
		for index,frame in pairs(tanks.tankFrames) do
			self:UpdateTankFrame(frame)
			if addon.db.profile.TankShowTarget then
				self:UpdateTankFrame(frame.target)
			end
			if addon.db.profile.TankShowTargetTarget then
				self:UpdateTankFrame(frame.tot)
			end
		end
		self.timer = 0
	end
end

function tanks:RosterUpdate()
	local raiders = GetNumRaidMembers()
	if raiders == 0 then return end
	local tankCount = 0
	local nameList = ""
	for i=1,raiders,1 do
		local name, _, _, _, _, _, _, _, _, role, _ = GetRaidRosterInfo(i);
		if role == "MAINTANK" then
			nameList = name..","..nameList
			tankCount = tankCount + 1
		end
	end
	for k,v in pairs(self.localTanks) do
		nameList = k..","..nameList
	end
	-- add PT namelist here
	if tankCount > 0 then
		TankHeader:SetScript("OnUpdate",function(frame,elapsed) tanks:OnUpdate(elapsed) end)
	else
		TankHeader:SetScript("OnUpdate",nil)
	end
	TankHeader:SetAttribute("nameList",nameList)
end

function tanks:SetupFrame(f)
	local db = addon.db.profile.TankOptions
	f:SetWidth(db.unitWidth)
	f:SetHeight(db.unitHeight)
	f:SetScale(db.scale)
	f:SetAlpha(db.alpha)
	f:SetBackdrop({
		{bgFile = "Interface/Tooltips/UI-Tooltip-Background", 
         insets = { left = 1, right = 1, top = 1, bottom = 1 }}
	})
	f:SetBackdropColor(0,0,0)
	-- Tank Statusbar
	f.bar = CreateFrame("StatusBar", nil, f)
	f.bar:SetMinMaxValues(0,100)
	--f.bar:SetStatusBarTexture(LSM:Fetch("status",db.barTexture,true))
	f.bar:SetStatusBarTexture([[Interface\TargetingFrame\UI-StatusBar]])
	f.bar:SetWidth(db.unitWidth)
	f.bar:SetPoint("LEFT",f,"LEFT",0,0)
	f.bar:SetStatusBarColor(1,0,0)
	f.bar:SetValue(100)
	f.bar:SetHeight(db.unitHeight)
	local barbg = f.bar:CreateTexture(nil,"BACKGROUND")
	barbg:SetWidth(db.unitWidth)
	barbg:SetHeight(db.unitHeight)
	barbg:SetTexture(0,0,0,0.3)
	barbg:SetPoint("LEFT",f,"LEFT",0,0)
	-- Tank Health
	f.health = f.bar:CreateFontString(nil, "OVERLAY","GameFontHighlightSmall")
	f.health:SetJustifyH("RIGHT")
	f.health:SetPoint("RIGHT",f,"RIGHT",-2,0)
	-- Tank Name
	f.name = f.bar:CreateFontString(nil, "OVERLAY","GameFontHighlightSmall")
	f.name:SetJustifyH("LEFT")
	f.name:SetPoint("LEFT",f,"LEFT",16,0)
	-- Raid Icon
	f.icon = f.bar:CreateTexture(nil, "OVERLAY")
	f.icon:SetTexture("Interface\\TargetingFrame\\UI-RaidTargetingIcons")
	f.icon:SetPoint("LEFT",f,"LEFT",0,0)
	f.icon:SetWidth(db.unitHeight)
	f.icon:SetHeight(db.unitHeight)
	f.icon:Hide()
end

function tanks:AddTank(unit)
	if not unit then return end
	if not UnitInRaid(unit) then return end
	if (GetPartyAssignment("MAINTANK",unit,1)) then return end
	if self.localTanks[unit] then return end
	self.localTanks[UnitName(unit)]=true
	self:RosterUpdate()
end
function tanks:RemoveTank(unit)
	ClearPartyAssignment("MAINTANK",unit,1)
end